from _devbuild.gen.id_kind_asdl import Id_t

from asdl import const  # For const.NO_INTEGER
from asdl import runtime
from asdl.runtime import (
  PrettyLeaf, ExternalLeaf, PrettyArray, PrettyNode,
  Color_TypeName, Color_StringConst, Color_OtherConst, Color_External,
  Color_UserType
)

from typing import Optional, List, Tuple, Dict, Any

class cmd_value_e(object):
  Argv = 1
  Assign = 2

class cmd_value_t(runtime.CompoundObj):
  pass

class cmd_value__Argv(cmd_value_t):
  tag = 1
  __slots__ = ('argv', 'arg_spids', 'block')

  def __init__(self, argv=None, arg_spids=None, block=None):
    # type: (Optional[List[str]], Optional[List[int]], Optional[Any]) -> None
    self.argv = argv or []
    self.arg_spids = arg_spids or []
    self.block = block or None

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('cmd_value.Argv')
    L = out_node.fields

    if self.argv:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.argv:
        x0.children.append(PrettyLeaf(i0, Color_StringConst))
      L.append(('argv', x0))

    if self.arg_spids:  # ArrayType
      x1 = PrettyArray()
      for i1 in self.arg_spids:
        x1.children.append(PrettyLeaf(str(i1), Color_OtherConst))
      L.append(('arg_spids', x1))

    if self.block is not None:  # MaybeType
      x2 = ExternalLeaf(self.block)
      L.append(('block', x2))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('cmd_value.Argv')
    L = out_node.fields
    if self.argv:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.argv:
        x0.children.append(PrettyLeaf(i0, Color_StringConst))
      L.append(('argv', x0))

    if self.arg_spids:  # ArrayType
      x1 = PrettyArray()
      for i1 in self.arg_spids:
        x1.children.append(PrettyLeaf(str(i1), Color_OtherConst))
      L.append(('arg_spids', x1))

    if self.block is not None:  # MaybeType
      x2 = ExternalLeaf(self.block)
      L.append(('block', x2))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class cmd_value__Assign(cmd_value_t):
  tag = 2
  __slots__ = ('builtin_id', 'argv', 'arg_spids', 'pairs')

  def __init__(self, builtin_id=None, argv=None, arg_spids=None, pairs=None):
    # type: (Optional[builtin_t], Optional[List[str]], Optional[List[int]], Optional[List[assign_arg]]) -> None
    self.builtin_id = builtin_id
    self.argv = argv or []
    self.arg_spids = arg_spids or []
    self.pairs = pairs or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('cmd_value.Assign')
    L = out_node.fields

    assert self.builtin_id is not None
    x0 = PrettyLeaf(self.builtin_id.name, Color_TypeName)
    L.append(('builtin_id', x0))

    if self.argv:  # ArrayType
      x1 = PrettyArray()
      for i1 in self.argv:
        x1.children.append(PrettyLeaf(i1, Color_StringConst))
      L.append(('argv', x1))

    if self.arg_spids:  # ArrayType
      x2 = PrettyArray()
      for i2 in self.arg_spids:
        x2.children.append(PrettyLeaf(str(i2), Color_OtherConst))
      L.append(('arg_spids', x2))

    if self.pairs:  # ArrayType
      x3 = PrettyArray()
      for i3 in self.pairs:
        x3.children.append(i3.PrettyTree())
      L.append(('pairs', x3))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('cmd_value.Assign')
    L = out_node.fields
    assert self.builtin_id is not None
    x0 = PrettyLeaf(self.builtin_id.name, Color_TypeName)
    L.append(('builtin_id', x0))

    if self.argv:  # ArrayType
      x1 = PrettyArray()
      for i1 in self.argv:
        x1.children.append(PrettyLeaf(i1, Color_StringConst))
      L.append(('argv', x1))

    if self.arg_spids:  # ArrayType
      x2 = PrettyArray()
      for i2 in self.arg_spids:
        x2.children.append(PrettyLeaf(str(i2), Color_OtherConst))
      L.append(('arg_spids', x2))

    if self.pairs:  # ArrayType
      x3 = PrettyArray()
      for i3 in self.pairs:
        x3.children.append(i3.AbbreviatedTree())
      L.append(('pairs', x3))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class cmd_value(object):
  Argv = cmd_value__Argv
  Assign = cmd_value__Assign

class part_value_e(object):
  String = 1
  Array = 2

class part_value_t(runtime.CompoundObj):
  pass

class part_value__String(part_value_t):
  tag = 1
  __slots__ = ('s', 'quoted', 'do_split')

  def __init__(self, s=None, quoted=None, do_split=None):
    # type: (Optional[str], Optional[bool], Optional[bool]) -> None
    self.s = s
    self.quoted = quoted
    self.do_split = do_split

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('part_value.String')
    L = out_node.fields

    x0 = PrettyLeaf(self.s, Color_StringConst)
    L.append(('s', x0))

    x1 = PrettyLeaf('T' if self.quoted else 'F', Color_OtherConst)
    L.append(('quoted', x1))

    x2 = PrettyLeaf('T' if self.do_split else 'F', Color_OtherConst)
    L.append(('do_split', x2))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('part_value.String')
    L = out_node.fields
    x0 = PrettyLeaf(self.s, Color_StringConst)
    L.append(('s', x0))

    x1 = PrettyLeaf('T' if self.quoted else 'F', Color_OtherConst)
    L.append(('quoted', x1))

    x2 = PrettyLeaf('T' if self.do_split else 'F', Color_OtherConst)
    L.append(('do_split', x2))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class part_value__Array(part_value_t):
  tag = 2
  __slots__ = ('strs',)

  def __init__(self, strs=None):
    # type: (Optional[List[str]]) -> None
    self.strs = strs or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('part_value.Array')
    L = out_node.fields

    if self.strs:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.strs:
        x0.children.append(PrettyLeaf(i0, Color_StringConst))
      L.append(('strs', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('part_value.Array')
    L = out_node.fields
    if self.strs:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.strs:
        x0.children.append(PrettyLeaf(i0, Color_StringConst))
      L.append(('strs', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class part_value(object):
  String = part_value__String
  Array = part_value__Array

class value_e(object):
  Undef = 1
  Str = 2
  MaybeStrArray = 3
  AssocArray = 4
  Obj = 5

class value_t(runtime.CompoundObj):
  pass

class value__Undef(value_t):
  tag = 1
  __slots__ = ()

  def __init__(self, ):
    # type: () -> None
    pass

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.Undef')
    L = out_node.fields

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.Undef')
    L = out_node.fields
    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class value__Str(value_t):
  tag = 2
  __slots__ = ('s',)

  def __init__(self, s=None):
    # type: (Optional[str]) -> None
    self.s = s

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.Str')
    L = out_node.fields

    x0 = PrettyLeaf(self.s, Color_StringConst)
    L.append(('s', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.Str')
    L = out_node.fields
    x0 = PrettyLeaf(self.s, Color_StringConst)
    L.append(('s', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class value__MaybeStrArray(value_t):
  tag = 3
  __slots__ = ('strs',)

  def __init__(self, strs=None):
    # type: (Optional[List[str]]) -> None
    self.strs = strs or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.MaybeStrArray')
    L = out_node.fields

    if self.strs:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.strs:
        x0.children.append(PrettyLeaf(i0, Color_StringConst))
      L.append(('strs', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.MaybeStrArray')
    L = out_node.fields
    if self.strs:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.strs:
        x0.children.append(PrettyLeaf(i0, Color_StringConst))
      L.append(('strs', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class value__AssocArray(value_t):
  tag = 4
  __slots__ = ('d',)

  def __init__(self, d=None):
    # type: (Optional[Any]) -> None
    self.d = d

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.AssocArray')
    L = out_node.fields

    x0 = ExternalLeaf(self.d)
    L.append(('d', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.AssocArray')
    L = out_node.fields
    x0 = ExternalLeaf(self.d)
    L.append(('d', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class value__Obj(value_t):
  tag = 5
  __slots__ = ('obj',)

  def __init__(self, obj=None):
    # type: (Optional[Any]) -> None
    self.obj = obj

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.Obj')
    L = out_node.fields

    x0 = ExternalLeaf(self.obj)
    L.append(('obj', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('value.Obj')
    L = out_node.fields
    x0 = ExternalLeaf(self.obj)
    L.append(('obj', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class value(object):
  Undef = value__Undef
  Str = value__Str
  MaybeStrArray = value__MaybeStrArray
  AssocArray = value__AssocArray
  Obj = value__Obj

class var_flags_t(runtime.SimpleObj):
  pass

class var_flags_e(object):
  Exported = var_flags_t(1, 'Exported')
  ReadOnly = var_flags_t(2, 'ReadOnly')

class scope_t(runtime.SimpleObj):
  pass

class scope_e(object):
  LocalOnly = scope_t(1, 'LocalOnly')
  GlobalOnly = scope_t(2, 'GlobalOnly')
  Dynamic = scope_t(3, 'Dynamic')

class lvalue_e(object):
  Named = 1
  Indexed = 2
  Keyed = 3
  ObjIndex = 4
  ObjAttr = 5

class lvalue_t(runtime.CompoundObj):
  pass

class lvalue__Named(lvalue_t):
  tag = 1
  __slots__ = ('name', 'spids')

  def __init__(self, name=None, spids=None):
    # type: (Optional[str], Optional[List[int]]) -> None
    self.name = name
    self.spids = spids or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.Named')
    L = out_node.fields

    x0 = PrettyLeaf(self.name, Color_StringConst)
    L.append(('name', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.Named')
    L = out_node.fields
    x0 = PrettyLeaf(self.name, Color_StringConst)
    L.append(('name', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class lvalue__Indexed(lvalue_t):
  tag = 2
  __slots__ = ('name', 'index', 'spids')

  def __init__(self, name=None, index=None, spids=None):
    # type: (Optional[str], Optional[int], Optional[List[int]]) -> None
    self.name = name
    self.index = index
    self.spids = spids or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.Indexed')
    L = out_node.fields

    x0 = PrettyLeaf(self.name, Color_StringConst)
    L.append(('name', x0))

    x1 = PrettyLeaf(str(self.index), Color_OtherConst)
    L.append(('index', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.Indexed')
    L = out_node.fields
    x0 = PrettyLeaf(self.name, Color_StringConst)
    L.append(('name', x0))

    x1 = PrettyLeaf(str(self.index), Color_OtherConst)
    L.append(('index', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class lvalue__Keyed(lvalue_t):
  tag = 3
  __slots__ = ('name', 'key', 'spids')

  def __init__(self, name=None, key=None, spids=None):
    # type: (Optional[str], Optional[str], Optional[List[int]]) -> None
    self.name = name
    self.key = key
    self.spids = spids or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.Keyed')
    L = out_node.fields

    x0 = PrettyLeaf(self.name, Color_StringConst)
    L.append(('name', x0))

    x1 = PrettyLeaf(self.key, Color_StringConst)
    L.append(('key', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.Keyed')
    L = out_node.fields
    x0 = PrettyLeaf(self.name, Color_StringConst)
    L.append(('name', x0))

    x1 = PrettyLeaf(self.key, Color_StringConst)
    L.append(('key', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class lvalue__ObjIndex(lvalue_t):
  tag = 4
  __slots__ = ('obj', 'index', 'spids')

  def __init__(self, obj=None, index=None, spids=None):
    # type: (Optional[Any], Optional[Any], Optional[List[int]]) -> None
    self.obj = obj
    self.index = index
    self.spids = spids or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.ObjIndex')
    L = out_node.fields

    x0 = ExternalLeaf(self.obj)
    L.append(('obj', x0))

    x1 = ExternalLeaf(self.index)
    L.append(('index', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.ObjIndex')
    L = out_node.fields
    x0 = ExternalLeaf(self.obj)
    L.append(('obj', x0))

    x1 = ExternalLeaf(self.index)
    L.append(('index', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class lvalue__ObjAttr(lvalue_t):
  tag = 5
  __slots__ = ('obj', 'attr', 'spids')

  def __init__(self, obj=None, attr=None, spids=None):
    # type: (Optional[Any], Optional[str], Optional[List[int]]) -> None
    self.obj = obj
    self.attr = attr
    self.spids = spids or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.ObjAttr')
    L = out_node.fields

    x0 = ExternalLeaf(self.obj)
    L.append(('obj', x0))

    x1 = PrettyLeaf(self.attr, Color_StringConst)
    L.append(('attr', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('lvalue.ObjAttr')
    L = out_node.fields
    x0 = ExternalLeaf(self.obj)
    L.append(('obj', x0))

    x1 = PrettyLeaf(self.attr, Color_StringConst)
    L.append(('attr', x1))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class lvalue(object):
  Named = lvalue__Named
  Indexed = lvalue__Indexed
  Keyed = lvalue__Keyed
  ObjIndex = lvalue__ObjIndex
  ObjAttr = lvalue__ObjAttr

class redirect_e(object):
  Path = 1
  FileDesc = 2
  HereDoc = 3

class redirect_t(runtime.CompoundObj):
  pass

class redirect__Path(redirect_t):
  tag = 1
  __slots__ = ('op_id', 'fd', 'filename', 'op_spid')

  def __init__(self, op_id=None, fd=None, filename=None, op_spid=None):
    # type: (Optional[Id_t], Optional[int], Optional[str], Optional[int]) -> None
    self.op_id = op_id
    self.fd = fd
    self.filename = filename
    self.op_spid = op_spid

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('redirect.Path')
    L = out_node.fields

    assert self.op_id is not None
    x0 = PrettyLeaf(self.op_id.name, Color_UserType)
    L.append(('op_id', x0))

    x1 = PrettyLeaf(str(self.fd), Color_OtherConst)
    L.append(('fd', x1))

    x2 = PrettyLeaf(self.filename, Color_StringConst)
    L.append(('filename', x2))

    x3 = PrettyLeaf(str(self.op_spid), Color_OtherConst)
    L.append(('op_spid', x3))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('redirect.Path')
    L = out_node.fields
    assert self.op_id is not None
    x0 = PrettyLeaf(self.op_id.name, Color_UserType)
    L.append(('op_id', x0))

    x1 = PrettyLeaf(str(self.fd), Color_OtherConst)
    L.append(('fd', x1))

    x2 = PrettyLeaf(self.filename, Color_StringConst)
    L.append(('filename', x2))

    x3 = PrettyLeaf(str(self.op_spid), Color_OtherConst)
    L.append(('op_spid', x3))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class redirect__FileDesc(redirect_t):
  tag = 2
  __slots__ = ('op_id', 'fd', 'target_fd', 'op_spid')

  def __init__(self, op_id=None, fd=None, target_fd=None, op_spid=None):
    # type: (Optional[Id_t], Optional[int], Optional[int], Optional[int]) -> None
    self.op_id = op_id
    self.fd = fd
    self.target_fd = target_fd
    self.op_spid = op_spid

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('redirect.FileDesc')
    L = out_node.fields

    assert self.op_id is not None
    x0 = PrettyLeaf(self.op_id.name, Color_UserType)
    L.append(('op_id', x0))

    x1 = PrettyLeaf(str(self.fd), Color_OtherConst)
    L.append(('fd', x1))

    x2 = PrettyLeaf(str(self.target_fd), Color_OtherConst)
    L.append(('target_fd', x2))

    x3 = PrettyLeaf(str(self.op_spid), Color_OtherConst)
    L.append(('op_spid', x3))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('redirect.FileDesc')
    L = out_node.fields
    assert self.op_id is not None
    x0 = PrettyLeaf(self.op_id.name, Color_UserType)
    L.append(('op_id', x0))

    x1 = PrettyLeaf(str(self.fd), Color_OtherConst)
    L.append(('fd', x1))

    x2 = PrettyLeaf(str(self.target_fd), Color_OtherConst)
    L.append(('target_fd', x2))

    x3 = PrettyLeaf(str(self.op_spid), Color_OtherConst)
    L.append(('op_spid', x3))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class redirect__HereDoc(redirect_t):
  tag = 3
  __slots__ = ('fd', 'body', 'op_spid')

  def __init__(self, fd=None, body=None, op_spid=None):
    # type: (Optional[int], Optional[str], Optional[int]) -> None
    self.fd = fd
    self.body = body
    self.op_spid = op_spid

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('redirect.HereDoc')
    L = out_node.fields

    x0 = PrettyLeaf(str(self.fd), Color_OtherConst)
    L.append(('fd', x0))

    x1 = PrettyLeaf(self.body, Color_StringConst)
    L.append(('body', x1))

    x2 = PrettyLeaf(str(self.op_spid), Color_OtherConst)
    L.append(('op_spid', x2))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('redirect.HereDoc')
    L = out_node.fields
    x0 = PrettyLeaf(str(self.fd), Color_OtherConst)
    L.append(('fd', x0))

    x1 = PrettyLeaf(self.body, Color_StringConst)
    L.append(('body', x1))

    x2 = PrettyLeaf(str(self.op_spid), Color_OtherConst)
    L.append(('op_spid', x2))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class redirect(object):
  Path = redirect__Path
  FileDesc = redirect__FileDesc
  HereDoc = redirect__HereDoc

class job_status_e(object):
  Process = 1
  Pipeline = 2

class job_status_t(runtime.CompoundObj):
  pass

class job_status__Process(job_status_t):
  tag = 1
  __slots__ = ('status',)

  def __init__(self, status=None):
    # type: (Optional[int]) -> None
    self.status = status

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('job_status.Process')
    L = out_node.fields

    x0 = PrettyLeaf(str(self.status), Color_OtherConst)
    L.append(('status', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('job_status.Process')
    L = out_node.fields
    x0 = PrettyLeaf(str(self.status), Color_OtherConst)
    L.append(('status', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class job_status__Pipeline(job_status_t):
  tag = 2
  __slots__ = ('statuses',)

  def __init__(self, statuses=None):
    # type: (Optional[List[int]]) -> None
    self.statuses = statuses or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('job_status.Pipeline')
    L = out_node.fields

    if self.statuses:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.statuses:
        x0.children.append(PrettyLeaf(str(i0), Color_OtherConst))
      L.append(('statuses', x0))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('job_status.Pipeline')
    L = out_node.fields
    if self.statuses:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.statuses:
        x0.children.append(PrettyLeaf(str(i0), Color_OtherConst))
      L.append(('statuses', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class job_status(object):
  Process = job_status__Process
  Pipeline = job_status__Pipeline

class span_t(runtime.SimpleObj):
  pass

class span_e(object):
  Black = span_t(1, 'Black')
  Delim = span_t(2, 'Delim')
  Backslash = span_t(3, 'Backslash')

class emit_t(runtime.SimpleObj):
  pass

class emit_e(object):
  Part = emit_t(1, 'Part')
  Delim = emit_t(2, 'Delim')
  Empty = emit_t(3, 'Empty')
  Escape = emit_t(4, 'Escape')
  Nothing = emit_t(5, 'Nothing')

class state_t(runtime.SimpleObj):
  pass

class state_e(object):
  Invalid = state_t(1, 'Invalid')
  Start = state_t(2, 'Start')
  DE_White1 = state_t(3, 'DE_White1')
  DE_Gray = state_t(4, 'DE_Gray')
  DE_White2 = state_t(5, 'DE_White2')
  Black = state_t(6, 'Black')
  Backslash = state_t(7, 'Backslash')

class char_kind_t(runtime.SimpleObj):
  pass

class char_kind_e(object):
  DE_White = char_kind_t(1, 'DE_White')
  DE_Gray = char_kind_t(2, 'DE_Gray')
  Black = char_kind_t(3, 'Black')
  Backslash = char_kind_t(4, 'Backslash')

class builtin_t(runtime.SimpleObj):
  pass

class builtin_e(object):
  NONE = builtin_t(1, 'NONE')
  READ = builtin_t(2, 'READ')
  ECHO = builtin_t(3, 'ECHO')
  PRINTF = builtin_t(4, 'PRINTF')
  SHIFT = builtin_t(5, 'SHIFT')
  CD = builtin_t(6, 'CD')
  PWD = builtin_t(7, 'PWD')
  PUSHD = builtin_t(8, 'PUSHD')
  POPD = builtin_t(9, 'POPD')
  DIRS = builtin_t(10, 'DIRS')
  EXPORT = builtin_t(11, 'EXPORT')
  READONLY = builtin_t(12, 'READONLY')
  LOCAL = builtin_t(13, 'LOCAL')
  DECLARE = builtin_t(14, 'DECLARE')
  TYPESET = builtin_t(15, 'TYPESET')
  UNSET = builtin_t(16, 'UNSET')
  SET = builtin_t(17, 'SET')
  SHOPT = builtin_t(18, 'SHOPT')
  TRAP = builtin_t(19, 'TRAP')
  UMASK = builtin_t(20, 'UMASK')
  SOURCE = builtin_t(21, 'SOURCE')
  DOT = builtin_t(22, 'DOT')
  EVAL = builtin_t(23, 'EVAL')
  EXEC = builtin_t(24, 'EXEC')
  WAIT = builtin_t(25, 'WAIT')
  JOBS = builtin_t(26, 'JOBS')
  FG = builtin_t(27, 'FG')
  BG = builtin_t(28, 'BG')
  COMPLETE = builtin_t(29, 'COMPLETE')
  COMPGEN = builtin_t(30, 'COMPGEN')
  COMPOPT = builtin_t(31, 'COMPOPT')
  COMPADJUST = builtin_t(32, 'COMPADJUST')
  TRUE = builtin_t(33, 'TRUE')
  FALSE = builtin_t(34, 'FALSE')
  COLON = builtin_t(35, 'COLON')
  TEST = builtin_t(36, 'TEST')
  BRACKET = builtin_t(37, 'BRACKET')
  GETOPTS = builtin_t(38, 'GETOPTS')
  COMMAND = builtin_t(39, 'COMMAND')
  TYPE = builtin_t(40, 'TYPE')
  HASH = builtin_t(41, 'HASH')
  HELP = builtin_t(42, 'HELP')
  HISTORY = builtin_t(43, 'HISTORY')
  BUILTIN = builtin_t(44, 'BUILTIN')
  ALIAS = builtin_t(45, 'ALIAS')
  UNALIAS = builtin_t(46, 'UNALIAS')
  REPR = builtin_t(47, 'REPR')
  PUSH = builtin_t(48, 'PUSH')
  USE = builtin_t(49, 'USE')
  ENV = builtin_t(50, 'ENV')
  FORK = builtin_t(51, 'FORK')
  OPTS = builtin_t(52, 'OPTS')
  JSON = builtin_t(53, 'JSON')

class effect_t(runtime.SimpleObj):
  pass

class effect_e(object):
  SpliceParts = effect_t(1, 'SpliceParts')
  Error = effect_t(2, 'Error')
  SpliceAndAssign = effect_t(3, 'SpliceAndAssign')
  NoOp = effect_t(4, 'NoOp')

class job_state_t(runtime.SimpleObj):
  pass

class job_state_e(object):
  Running = job_state_t(1, 'Running')
  Done = job_state_t(2, 'Done')
  Stopped = job_state_t(3, 'Stopped')

class word_style_t(runtime.SimpleObj):
  pass

class word_style_e(object):
  Expr = word_style_t(1, 'Expr')
  Unquoted = word_style_t(2, 'Unquoted')
  DQ = word_style_t(3, 'DQ')
  SQ = word_style_t(4, 'SQ')

class assign_arg(runtime.CompoundObj):
  tag = 1000
  __slots__ = ('lval', 'rval', 'spid')

  def __init__(self, lval=None, rval=None, spid=None):
    # type: (Optional[lvalue_t], Optional[value_t], Optional[int]) -> None
    self.lval = lval
    self.rval = rval or None
    self.spid = spid

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('assign_arg')
    L = out_node.fields

    assert self.lval is not None
    x0 = self.lval.PrettyTree()
    L.append(('lval', x0))

    if self.rval is not None:  # MaybeType
      x1 = self.rval.PrettyTree()
      L.append(('rval', x1))

    x2 = PrettyLeaf(str(self.spid), Color_OtherConst)
    L.append(('spid', x2))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('assign_arg')
    L = out_node.fields
    assert self.lval is not None
    x0 = self.lval.AbbreviatedTree()
    L.append(('lval', x0))

    if self.rval is not None:  # MaybeType
      x1 = self.rval.AbbreviatedTree()
      L.append(('rval', x1))

    x2 = PrettyLeaf(str(self.spid), Color_OtherConst)
    L.append(('spid', x2))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class arg_vector(runtime.CompoundObj):
  tag = 1001
  __slots__ = ('strs', 'spids')

  def __init__(self, strs=None, spids=None):
    # type: (Optional[List[str]], Optional[List[int]]) -> None
    self.strs = strs or []
    self.spids = spids or []

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('arg_vector')
    L = out_node.fields

    if self.strs:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.strs:
        x0.children.append(PrettyLeaf(i0, Color_StringConst))
      L.append(('strs', x0))

    if self.spids:  # ArrayType
      x1 = PrettyArray()
      for i1 in self.spids:
        x1.children.append(PrettyLeaf(str(i1), Color_OtherConst))
      L.append(('spids', x1))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('arg_vector')
    L = out_node.fields
    if self.strs:  # ArrayType
      x0 = PrettyArray()
      for i0 in self.strs:
        x0.children.append(PrettyLeaf(i0, Color_StringConst))
      L.append(('strs', x0))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

class cell(runtime.CompoundObj):
  tag = 1002
  __slots__ = ('val', 'exported', 'readonly')

  def __init__(self, val=None, exported=None, readonly=None):
    # type: (Optional[value_t], Optional[bool], Optional[bool]) -> None
    self.val = val
    self.exported = exported
    self.readonly = readonly

  def PrettyTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('cell')
    L = out_node.fields

    assert self.val is not None
    x0 = self.val.PrettyTree()
    L.append(('val', x0))

    x1 = PrettyLeaf('T' if self.exported else 'F', Color_OtherConst)
    L.append(('exported', x1))

    x2 = PrettyLeaf('T' if self.readonly else 'F', Color_OtherConst)
    L.append(('readonly', x2))

    return out_node

  def _AbbreviatedTree(self):
    # type: () -> PrettyNode
    out_node = PrettyNode('cell')
    L = out_node.fields
    assert self.val is not None
    x0 = self.val.AbbreviatedTree()
    L.append(('val', x0))

    x1 = PrettyLeaf('T' if self.exported else 'F', Color_OtherConst)
    L.append(('exported', x1))

    x2 = PrettyLeaf('T' if self.readonly else 'F', Color_OtherConst)
    L.append(('readonly', x2))

    return out_node

  def AbbreviatedTree(self):
    # type: () -> PrettyNode
    return self._AbbreviatedTree()

